#################################
###PARABOLIC CAMBRIAN LATTICES###
#################################
    
def alpha_permutations(alpha):
	r"""
	Computes the elements of a parabolic quotient of a symmetric group with respect to the integer composition 'alpha'.

	Let `alpha=(a_1,a_2,\ldots,a_r)`, and define `s_i=a_1+a_2+\cdots+a_i`.  This function constructs the elements of the parabolic quotient of the symmetric group of degree `a_1+a_2+\cdots+a_r` by its subgroup generated by the adjacent transpositions `(i,i+1)` for `i \notin \{s_1,s_2,\ldots,s_r\}`.  These are the permutations `w` for which holds `w(i)<w(i+1)` if and only if `i \notin \{s_1,s_2,\ldots,s_r\}`.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: [Permutation(w) for w in alpha_permutations((1,2,1))]
		[[1, 2, 3, 4],
		 [2, 1, 4, 3],
		 [1, 3, 4, 2],
		 [3, 1, 2, 4],
		 [3, 2, 4, 1],
		 [4, 1, 3, 2],
		 [1, 2, 4, 3],
		 [2, 1, 3, 4],
		 [3, 1, 4, 2],
		 [4, 2, 3, 1],
		 [4, 1, 2, 3],
		 [2, 3, 4, 1]]
	"""
	return [w for w in SymmetricGroup(sum(alpha)) if all([is_increasing(x) for x in split_permutation(Permutation(w),alpha)])]

def alpha_231_avoiding_permutations(alpha):
	r"""
	Constructs the set of (alpha,pattern)-avoiding permutations, where 'pattern' is either 231 or 312.  See also Section~3 of [MW18]_.
	
	An alpha-permutation `w` has an *(alpha,231)-pattern* if there exist three indices `1 \leq i < j < k \leq n` such that the following holds:
	
	1. `i,j,k` belong to different alpha-regions
	
	2. It holds that `w(k) < w(i) < w(j)`.
	
	3. It holds that `w(i) = w(k) + 1`.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: [Permutation(w) for w in alpha_231_avoiding_permutations((1,2,1))]
		[[1, 2, 3, 4],
		 [2, 1, 4, 3],
		 [1, 3, 4, 2],
		 [3, 1, 2, 4],
		 [3, 2, 4, 1],
		 [4, 1, 3, 2],
		 [1, 2, 4, 3],
		 [2, 1, 3, 4],
		 [4, 2, 3, 1],
		 [4, 1, 2, 3]]
		 
	REFERENCES::
	
		.. [MW18] Henri M\"uhle, Nathan Williams.  
		   *Tamari Lattices for Parabolic Quotients of the Symmetric Group*
		   arXiv:1804.02761, 2018.
    """
	aperms = alpha_permutations(alpha)
	aregs = create_alpha_regions(alpha)
	aavoid = []    
	for w in aperms:
		p = Permutation(w)
		has_pattern = false 
		positions = p.pattern_positions([2,3,1])
		if len(positions)>0:	#check if p has a pattern
			for pos in positions:
				ipos = [a+1 for a in pos]
				if max([len(Set(reg).intersection(Set(ipos))) for reg in aregs])>1:	#the pattern is not spread out over three alpha-regions
					continue
				if p[pos[0]]>p[pos[2]]+1:	#the indices i and k do not form a descent
					continue
				j = region_index(alpha,ipos[1])
				has_pattern = true
				break
		if not has_pattern:
			aavoid.append(w)
	return aavoid

def alpha_tamari_lattice(alpha):
	r"""
	Constructs the alpha-Tamari lattice.  See also Section~3 of [MW18]_.
	
	The *alpha-Tamari lattice* is the lattice of all (alpha,231)-avoiding permutations under left-weak order.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_tamari_lattice((1,2,1))
		Finite lattice containing 10 elements
	
	REFERENCES::
	
		.. [MW18] Henri M\"uhle, Nathan Williams.  
		   *Tamari Lattices for Parabolic Quotients of the Symmetric Group*
		   arXiv:1804.02761, 2018.
	"""
	parabolics = alpha_231_avoiding_permutations(alpha)
	order = lambda u,v:u.weak_le(v)
	return LatticePoset([parabolics,order])
	
def enumerate_alpha_tamari_intervals(alpha):
	r"""
	Enumerates the intervals in the alpha-Tamari lattice.
	
	:param alpha: an integer composition.

	EXAMPLES::
	
		sage: enumerate_alpha_tamari_intervals((1,2,1))
		42
		sage: enumerate_alpha_tamari_intervals((1,1,1,1))
		68
	"""
	return len(alpha_tamari_lattice(alpha).relations())

def alpha_231_avoiding_permutation_to_partition(w):
	r"""
	Constructs the noncrossing alpha-partition associated with the (alpha,231)-avoiding permutation 'w'.
	
	For every integer composition 'alpha', the sets of (alpha,231)-avoiding permutations and noncrossing alpha-partitions are in bijection.  The map from permutations to partitions is given by grouping indices connected by descents to blocks.
	
	:param w: an (alpha,231)-avoiding ermutation
	
	EXAMPLES::
		
		sage: A = alpha_231_avoiding_permutations((1,2,1))
		sage: [alpha_231_avoiding_permutation_to_partition(w) for w in A]
		[{{1}, {2}, {3}, {4}},
		 {{1, 2}, {3, 4}},
		 {{1}, {2, 4}, {3}},
		 {{1, 3}, {2}, {4}},
		 {{1, 2, 4}, {3}},
		 {{1, 3, 4}, {2}},
		 {{1}, {2}, {3, 4}},
		 {{1, 2}, {3}, {4}},
		 {{1, 3}, {2, 4}},
		 {{1, 4}, {2}, {3}}]

	"""
	p = Permutation(w)
	bumps = []
	for i in range(len(p)):
		for j in range(i+1,len(p)):
			if p[i]==p[j]+1:
				bumps.append((i+1,j+1))
	G = Graph()
	[G.add_vertex(i) for i in range(1,len(p)+1)]
	for b in bumps:
		G.add_edge(b[0],b[1])
	return SetPartition(G.connected_components())
