#########################################
###JOIN REPRESENTATIONS AND SHARD SETS###
#########################################
	
def alpha_join_representation(w):
	r"""
	Constructs the canonical join representation of an element of some alpha-Tamari lattice.
	
	The canonical join representation of an element of an alpha-Tamari lattice is simply its descent set.
	
	:param x: an alpha-permutation
	
	EXAMPLES::
	
		sage: [alpha_join_representation(w) for w in alpha_231_avoiding_permutations((2,2))]
		[[], [(1, 4)], [(2, 4)], [(1, 3)], [(2, 3)], [(1, 3), (2, 4)]]
	"""
	n = w.parent().degree()
	join_rep = []
	for i in range(1,n+1):
		for j in range(i+1,n+1):
			if w(j)==w(i)-1:
				join_rep.append(tuple([i,j]))
	return join_rep
		
def alpha_join_complex(alpha):
	r"""
	Constructs the canonical join complex of the alpha-Tamari lattice as defined in Section~2 of [Rea15]_.
	
	The canonical join complex of a join-semidistributive lattice `L` is the simplicial complex whose faces are the canonical join-representations of the elements of `L`.  It was introduced in [Rea15]_ and was further investigated in [Bar19]_.
	
	:param alpha: an integer composition.
		
	EXAMPLES::
	
		sage: D = alpha_join_complex((1,2,1)); D
		Simplicial complex with 5 vertices and 5 facets
		sage: D.facets()
		{((1, 4),), ((1, 3), (3, 4)), ((1, 2), (3, 4)), ((1, 2), (2, 4)), ((1, 3), (2, 4))}

	REFERENCES::
	
		.. [Rea15] Nathan Reading.  
		   *Noncrossing Arc Diagrams and Canonical Join Representations*
		   SIAM J. Discrete Math., *29*(2), pages 736-750. 2015.
		   
		.. [Bar19] Emily Barnard.
		   *The Canonical Join Complex*
		   Electron. J. Combin., *26*. 2019.
	"""
	return SimplicialComplex([alpha_join_representation(x) for x in alpha_231_avoiding_permutations(alpha)])

def alpha_core_label_set(C,w):
	r"""
	Constructs the core label set of the given element of the alpha-Tamari lattice.
	
	A cover relation `u \lessdot v` in the alpha-Cambrian lattice is labeled by a pair `(i,j)`, where `(i,j) \in \des(v) \setminus \inv(u)`, i.e. by the unique descent of `v` that is not an inversion of `u`.  The core label set of an element of the alpha-Tamari lattice is the set of all such labels that occur in the interval `[x',x]`, where `x'` is the meet of all lower covers of `x`.
	
	:param C: an alpha-Tamari lattice
	:param w: an element of `C`.
	
	EXAMPLES::
	
		sage: C = alpha_tamari_lattice((2,2,1)); C
		Finite lattice containing 22 elements
		sage: [alpha_core_label_set(C,w) for w in C]
		[{},
		 {(4, 5)},
		 {(3, 5)},
		 {(2, 3)},
		 {(4, 5), (2, 3)},
		 {(2, 3), (3, 5)},
		 {(2, 4)},
		 {(2, 5)},
		 {(4, 5), (2, 5), (2, 4)},
		 {(2, 4), (3, 5)},
		 {(1, 3)},
		 {(4, 5), (1, 3)},
		 {(1, 3), (3, 5)},
		 {(1, 3), (2, 4)},
		 {(2, 5), (1, 3)},
		 {(4, 5), (2, 5), (1, 3), (2, 4)},
		 {(1, 3), (2, 4), (3, 5)},
		 {(1, 4)},
		 {(2, 5), (1, 4)},
		 {(1, 5)},
		 {(4, 5), (1, 5), (1, 4)},
		 {(1, 4), (3, 5)}]
	"""
	assert(w in C)
	
	n = w.parent().degree()
	ww = C.meet(C.lower_covers(w))
	Q = C.subposet(C.interval(ww,w))
	shards = []
	for cov in Q.cover_relations():
		#find inversions of cov[0]
		invs = []
		for i in range(1,n+1):
			for j in range(i+1,n+1):
				if cov[0](i)>cov[0](j):
					invs.append(tuple([i,j]))
		L = list(Set(alpha_join_representation(cov[1])).difference(Set(invs)))
		assert(len(L)==1)
		lab = L[0]
		shards.append(lab)
	return Set(shards)

def alpha_core_label_order(alpha):
	r"""
	Constructs the core label order of the alpha-Tamari lattice.
	
	The core label order of a congruence-uniform lattice is the family of core label sets ordered by inclusion.
	
	:param alpha: an integer composition.
		
	EXAMPLES::
	
		sage: P = alpha_core_label_order((1,2,1)); P
		Finite poset containing 10 elements
		sage: P.is_meet_semilattice()
		True
		sage: P.is_isomorphic(alpha_partition_poset((1,2,1),noncrossing=true))
		False
		sage: alpha_core_label_order((2,1,2)).is_isomorphic(alpha_partition_poset((2,1,2),noncrossing=true))
		True
	"""
	C = alpha_tamari_lattice(alpha)
	core_label_sets = [alpha_core_label_set(C,w) for w in C]
	incl = lambda A,B: A.issubset(B)
	return Poset([core_label_sets,incl])
