######################################
###PARABOLIC NONCROSSING PARTITIONS###
######################################

def is_parabolic(P,alpha):
	r"""
	Checks whether the given set partition is an `alpha`-partition.
	
	:param P: a set partition.
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: P = SetPartition([[1, 4, 5, 8], [2, 3], [6], [7, 9]])
		sage: is_parabolic(P, (2, 2, 3, 2))
		True
		sage: is_parabolic(P, (4, 5))
		False
	"""
	regions = create_alpha_regions(alpha)
	parabolic = true
	
	for B in P.arcs():
		for R in regions:
			if B[0] in R and B[1] in R:
				parabolic = false
				break
	return parabolic
	
def is_noncrossing(P,alpha):
	r"""
	Checks whether the given alpha-partition is noncrossing.
	
	:param P: a set partition.
	:param alpha: an integer partition.
	
	EXAMPLES::
		sage: P = SetPartition([[1, 4, 5, 8], [2, 3], [6], [7, 9]])
		sage: is_noncrossing(P,(2,2,3,2))
		False
		sage: is_noncrossing(P,(1,1,2,3,2))
		True
	"""
	regions = create_alpha_regions(alpha)
	bumps = P.arcs()
	crossing_one = false
	crossing_two = false
	
	for B1 in bumps:
		for B2 in bumps:
			if B1[0]<B2[0] and B2[0]<B1[1] and B1[1]<B2[1]:
				gr_1 = false
				gr_2 = false
				for r in regions:
					if B1[0] in r and B2[0] in r:
						gr_1 = true
					if B1[1] in r and B2[0] in r:
						gr_2 = true
				if (not gr_1 and not gr_2) or (gr_1 and gr_2):
					crossing_one = true
					break
			if B1[0]<B2[0] and B2[0]<B2[1] and B2[1]<B1[1]:
				for r in regions:
					if B1[0] in r and B2[0] in r:
						crossing_two = true
						break
	if crossing_one or crossing_two:
		return false
	else:
		return true
		
def alpha_partitions(alpha,noncrossing=true):
	r"""
	Creates all (noncrossing) parabolic set partitions as defined in Section~4 of [MW18]_.
	
	In a set partition of `\{1,2,\ldots,n\}`, a *bump* is a pair of consecutive entries of a block.  We split the ground set into consecutive intervals according to the integer composition 'alpha' into so-called *regions*.  An *alpha-partition* is a set partition in which no bump connects entries in the same region.  

	An alpha-partition is *noncrossing* if the following conditions hold. 
	
	1. For two bumps `(a,b)` and `(c,d)` with `a<c<b<d` holds that either `a` and `c` lie in the same region or `b` and `c` lie in the same region.
	
	2. For two bumps `(a,b)` and `(c,d)` with `a<c<d<b` holds that `a` and `c` lie in different regions.
	
	:param alpha: an integer composition.
	:param noncrossing: a boolean variable.

	EXAMPLES::
	
		sage: Set(alpha_partitions((1, 2, 2), noncrossing=false)).difference(Set(alpha_partitions((1, 2, 2), noncrossing=true)))
		{{{1, 4}, {2}, {3, 5}}, {{1}, {2, 5}, {3, 4}}, {{1, 4}, {2, 5}, {3}}, {{1, 3, 4}, {2, 5}}, {{1, 2, 5}, {3, 4}}}
		sage: alpha_partitions((1,2,1))
		[{{1, 3, 4}, {2}},
		 {{1, 2, 4}, {3}},
		 {{1, 2}, {3, 4}},
		 {{1, 3}, {2, 4}},
		 {{1}, {2}, {3, 4}},
		 {{1}, {2, 4}, {3}},
		 {{1, 4}, {2}, {3}},
		 {{1, 3}, {2}, {4}},
		 {{1, 2}, {3}, {4}},
		 {{1}, {2}, {3}, {4}}]
	
	REFERENCES::
	
		.. [MW18] Henri M\"uhle, Nathan Williams.  
		   *Tamari Lattices for Parabolic Quotients of the Symmetric Group*
		   arXiv:1804.02761, 2018.
	"""
	n = sum(alpha)
	P = SetPartitions(n)
	parabolics = []
	for X in P:
		if not is_parabolic(X,alpha):
			continue
		if noncrossing:
			if not is_noncrossing(X,alpha):
				continue
		parabolics.append(X)
	return parabolics

def alpha_partition_poset(alpha,noncrossing=true):
	r"""
	Constructs the poset of parabolic (noncrossing) partitions under (dual) refinement order.
	
	:param alpha: an integer composition.
	:param noncrossing: a boolean variable.
	
	EXAMPLES::
	
		sage: P = alpha_partition_poset((1, 2, 1)); P
		Finite poset containing 10 elements
		sage: P.is_meet_semilattice()
		True

	"""
	X = alpha_partitions(alpha,noncrossing)
	dref = lambda u,v:v in u.coarsenings()
	return Poset([X,dref])
